@@ -26,7 +26,7 @@ dm_bind <- function(..., repair = "check_unique", quiet = FALSE) {
   }
 
   # repair table names
-  table_names <- map(dms, src_tbls) %>% flatten_chr()
+  table_names <- map(dms, src_tbls_impl) %>% flatten_chr()
   new_table_names <- repair_names_vec(table_names, repair, quiet)
   # need to individually rename tables for each `dm`
   ntables_dms <- map(dms, length)
