@@ -34,8 +34,8 @@ get_by <- function(dm, lhs_name, rhs_name) {
   if (length(lhs_col) > 1 || length(rhs_col) > 1) abort_no_cycles(create_graph_from_dm(dm))
   # Construct a `by` argument of the form `c("lhs_col[1]" = "rhs_col[1]", ...)`
   # as required by `*_join()`
-  by <- rhs_col
-  names(by) <- lhs_col
+  by <- get_key_cols(rhs_col)
+  names(by) <- get_key_cols(lhs_col)
   by
 }
 
