@@ -3,7 +3,7 @@ replay_html_dm_draw <- function(x, ...) {
 }
 
 # Use custom name to avoid clash if pkgload is active
-print_grViz <- function(x, ...) {
+pkgdown_print_grViz <- function(x, ...) {
   structure(DiagrammeRsvg::export_svg(x), class = "dm_draw")
 }
 
@@ -15,6 +15,5 @@ register_pkgdown_methods <- function() {
   stopifnot(rlang::is_installed(c("DiagrammeR", "DiagrammeRsvg")))
   # For dev pkgdown
   vctrs::s3_register("downlit::replay_html", "dm_draw", replay_html_dm_draw)
-  vctrs::s3_register("pkgdown::replay_html", "dm_draw", replay_html_dm_draw)
-  vctrs::s3_register("base::print", "grViz", print_grViz)
+  vctrs::s3_register("pkgdown::pkgdown_print", "grViz", pkgdown_print_grViz)
 }
