@@ -16,8 +16,8 @@ skip_if_local_src <- function(src = my_test_src()) {
   skip_if_not_installed("dbplyr")
 }
 
-skip_if_src <- function(name) {
-  if (my_test_src_name == name) skip(paste0("does not work on ", name))
+skip_if_src <- function(...) {
+  if (my_test_src_name %in% c(...)) skip(glue::glue("does not work on {commas(tick(c(...)))}"))
 }
 
 skip_if_src_not <- function(...) {
