@@ -42,6 +42,7 @@ copy_to_my_test_src <- function(rhs, lhs) {
     rhs
   } else if (is_dm(rhs)) {
     # We want all dm operations to work with key constraints on the database
+    # (except for bad_dm)
     # message(name)
     suppressMessages(copy_dm_to(src, rhs))
   } else if (inherits(rhs, "list")) {
@@ -100,8 +101,8 @@ test_src_frame <- function(...) {
     temporary <- TRUE
   }
 
-  copy_to(src, df, name = name, temporary = temporary)
-  tbl(src, name)
+  out <- copy_to(src, df, name = name, temporary = temporary)
+  out
 }
 
 # for examine_cardinality...() ----------------------------------------------
@@ -389,12 +390,6 @@ dm_for_disambiguate %<-% {
     dm_add_fk(iris_2, key, iris_1)
 }
 
-dm_for_disambiguate_2 %<-% {
-  as_dm(list(iris_1 = iris_1_dis(), iris_2 = iris_2_dis(), iris_3 = iris_3_dis())) %>%
-    dm_add_pk(iris_1, key) %>%
-    dm_add_fk(iris_2, iris_2.key, iris_1)
-}
-
 # star schema data model for testing `dm_flatten_to_tbl()` ------
 
 fact %<-% tibble(
@@ -410,7 +405,8 @@ fact %<-% tibble(
     "ill-advised",
     "jitter"
   ),
-  dim_1_key = 14:5,
+  dim_1_key_1 = 14:5,
+  dim_1_key_2 = LETTERS[14:5],
   dim_2_key = letters[3:12],
   dim_3_key = LETTERS[24:15],
   dim_4_key = 7:16,
@@ -425,7 +421,8 @@ fact_clean %<-% {
 }
 
 dim_1 %<-% tibble(
-  dim_1_pk = 1:20,
+  dim_1_pk_1 = 1:20,
+  dim_1_pk_2 = LETTERS[1:20],
   something = letters[3:22]
 )
 dim_1_clean %<-% {
@@ -468,11 +465,11 @@ dm_for_flatten %<-% {
     dim_3 = dim_3(),
     dim_4 = dim_4()
   )) %>%
-    dm_add_pk(dim_1, dim_1_pk) %>%
+    dm_add_pk(dim_1, c(dim_1_pk_1, dim_1_pk_2)) %>%
     dm_add_pk(dim_2, dim_2_pk) %>%
     dm_add_pk(dim_3, dim_3_pk) %>%
     dm_add_pk(dim_4, dim_4_pk) %>%
-    dm_add_fk(fact, dim_1_key, dim_1) %>%
+    dm_add_fk(fact, c(dim_1_key_1, dim_1_key_2), dim_1) %>%
     dm_add_fk(fact, dim_2_key, dim_2) %>%
     dm_add_fk(fact, dim_3_key, dim_3) %>%
     dm_add_fk(fact, dim_4_key, dim_4)
@@ -480,7 +477,7 @@ dm_for_flatten %<-% {
 
 result_from_flatten %<-% {
   fact_clean() %>%
-    left_join(dim_1_clean(), by = c("dim_1_key" = "dim_1_pk")) %>%
+    left_join(dim_1_clean(), by = c("dim_1_key_1" = "dim_1_pk_1", "dim_1_key_2" = "dim_1_pk_2")) %>%
     left_join(dim_2_clean(), by = c("dim_2_key" = "dim_2_pk")) %>%
     left_join(dim_3_clean(), by = c("dim_3_key" = "dim_3_pk")) %>%
     left_join(dim_4_clean(), by = c("dim_4_key" = "dim_4_pk"))
@@ -488,30 +485,31 @@ result_from_flatten %<-% {
 
 # 'bad' dm (no ref. integrity) for testing dm_flatten_to_tbl() --------
 
-tbl_1 %<-% tibble(a = as.integer(c(1, 2, 4, 5)), b = a)
-tbl_2 %<-% tibble(id = 1:2, c = letters[1:2])
-tbl_3 %<-% tibble(id = 2:4, d = letters[2:4])
+tbl_1 %<-% tibble(a = as.integer(c(1, 2, 4, 5, NA)), x = LETTERS[3:7], b = a)
+tbl_2 %<-% tibble(id = c(1:3, 3), x = LETTERS[c(3:5, 5)], c = letters[1:4])
+tbl_3 %<-% tibble(id = c(2:4, 4), d = letters[2:5])
 
-bad_dm %<-% {
-  as_dm(list(tbl_1 = tbl_1(), tbl_2 = tbl_2(), tbl_3 = tbl_3())) %>%
-    dm_add_pk(tbl_2, id) %>%
+bad_dm_base %<-% {
+  as_dm(list(tbl_1 = tbl_1(), tbl_2 = tbl_2(), tbl_3 = tbl_3()))
+}
+
+# avoid copying constraints for invalid dm
+bad_dm %<--% {
+  bad_dm_base() %>%
+    dm_add_pk(tbl_2, c(id, x)) %>%
     dm_add_pk(tbl_3, id) %>%
-    dm_add_fk(tbl_1, a, tbl_2) %>%
+    dm_add_fk(tbl_1, c(a, x), tbl_2) %>%
     dm_add_fk(tbl_1, b, tbl_3)
 }
 
 dm_nycflights_small_base %<-% {
-  dm(
-    flights =
-      nycflights13::flights %>%
-        slice(1:800),
-    planes = nycflights13::planes,
-    airlines = nycflights13::airlines,
-    airports = nycflights13::airports,
-    weather =
-      nycflights13::weather %>%
-        slice(1:800)
-  )
+  airlines <- nycflights13::airlines
+  airports <- nycflights13::airports
+  planes <- nycflights13::planes
+  flights <- flights_subset()
+  weather <- weather_subset()
+
+  dm(airlines, airports, flights, planes, weather)
 }
 
 # Do not add PK and FK constraints to the database
@@ -525,6 +523,17 @@ dm_nycflights_small %<--% {
     dm_add_fk(flights, dest, airports)
 }
 
+dm_nycflights_small_cycle %<--% {
+  dm_nycflights_small() %>%
+    dm_add_fk(flights, origin, airports)
+}
+
+nyc_comp %<--% {
+  dm_nycflights_small() %>%
+    dm_add_pk(weather, c(origin, time_hour)) %>%
+    dm_add_fk(flights, c(origin, time_hour), weather)
+}
+
 zoomed_dm <- function() dm_zoom_to(dm_for_filter(), tf_2)
 zoomed_dm_2 <- function() dm_zoom_to(dm_for_filter(), tf_3)
 
