@@ -5,5 +5,5 @@ get_all_keys <- function(dm, table_name) {
     filter(child_table == table_name) %>%
     pull(child_fk_cols)
   pk <- dm_get_pk_impl(dm, table_name)
-  set_names(unique(c(pk, fks)))
+  new_keys(unique(c(pk, fks)))
 }
