@@ -59,7 +59,7 @@ test_that("output", {
     abort_only_for_local_src(mtcars)
     abort_parameter_not_correct_class("number", correct_class = "numeric", class = "logical")
     abort_parameter_not_correct_length("length_1_parameter", 1, letters[1:26])
-    warn_if_arg_not("NULL", "dbms_dependent_arg")
+    warn_if_arg_not("NULL", c("MSSQL", "Postgres"), arg_name = "dbms_dependent_arg")
     abort_schema_exists("silhouette")
     abort_schema_exists("silhouette", "exhibition")
     abort_no_schema_exists("table_1")
