@@ -117,8 +117,9 @@ test_that("cdm_get_src() works", {
     class = "is_not_dm"
   )
 
+  skip_if_local_src()
   expect_identical(
-    class(dm_get_src(dm_for_filter())),
+    class(cdm_get_src(dm_for_filter())),
     class(my_test_src())
   )
 })
@@ -200,7 +201,7 @@ test_that("other FK functions work", {
 
   expect_identical(
     cdm_get_all_fks(dm_for_filter()) %>%
-      mutate(child_fk_cols = new_keys(child_fk_cols)),
+      mutate(child_fk_cols = new_keys(child_fk_cols), parent_pk_cols = new_keys(parent_pk_cols)),
     dm_get_all_fks(dm_for_filter())
   )
 
@@ -303,8 +304,9 @@ test_that("other PK functions work", {
   )
 
   expect_identical(
-    cdm_get_all_pks(dm_for_filter()),
-    dm_get_all_pks_impl(dm_for_filter())
+    cdm_get_all_pks(dm_for_filter()) %>%
+      mutate(pk_col = new_keys(pk_col)),
+    dm_get_all_pks(dm_for_filter())
   )
 
   expect_equivalent_dm(
@@ -340,13 +342,13 @@ test_that("dm_select() and dm_rename() work", {
   local_options(lifecycle_verbosity = "quiet")
 
   expect_equivalent_tbl(
-    cdm_select(dm_for_filter(), tf_1, a_new = a) %>% tbl("tf_1"),
-    dm_select(dm_for_filter(), tf_1, a_new = a) %>% tbl("tf_1")
+    cdm_select(dm_for_filter(), tf_1, a_new = a) %>% tbl_impl("tf_1"),
+    dm_select(dm_for_filter(), tf_1, a_new = a) %>% tbl_impl("tf_1")
   )
 
   expect_equivalent_tbl(
-    cdm_rename(dm_for_filter(), tf_1, a_new = a) %>% tbl("tf_1"),
-    dm_rename(dm_for_filter(), tf_1, a_new = a) %>% tbl("tf_1")
+    cdm_rename(dm_for_filter(), tf_1, a_new = a) %>% tbl_impl("tf_1"),
+    dm_rename(dm_for_filter(), tf_1, a_new = a) %>% tbl_impl("tf_1")
   )
 })
 
